#include <stdlib.h>
#include <stdio.h>

#include <ds2io.h>
#include <ds2_cpu.h>
#include <fs_api.h>

#include <ds2_malloc.h>

#include "myds2_timer.h"
#include "myds2_gfx.h"

#include "main.h"
#include "menu.h"
#include "intro.h"

gamecfg GameConf;


int main(int argc, char** argv) {
  srand(getSysTime());

  system_loadcfg("/A7800/A7800.cfg");
  ds2_setVolume(GameConf.sndLevel);
  ds2_setCPUclocklevel(10);
  printf("Clock=%dMhz\n",300);
  printf("begin emulation !\n");

  // Intro and main screen
  intro_logo();  
  dsInitScreenMain();
  etatEmu = A7800_MENUINIT;
  
  //load rom file via args if a rom path is supplied
  if(argc > 1) {
    dsShowScreenMain();
    dsLoadGame(argv[1]);
    etatEmu = A7800_PLAYINIT;
  }
  
  // Main loop of emulation
  dsMainLoop();
  
  // Free memory to be correct 
  dsFreeEmu();
  
  mdelay(3000);

  ds2_plug_exit();

  return 0;
}

void system_loadcfg(char *cfg_name) {
  FILE *fd;

  fd = fopen(cfg_name, "rb");
  if (fd != NULL) {
    fread(&GameConf, 1, sizeof(GameConf),fd); 
    fclose(fd);
  }
  else {
    // UP  DOWN  LEFT RIGHT  A  B   X  Y    R  L     START  SELECT
    // 2,    3,    1,    0,  4, 5, 12, 12, 15, 16,     14,      13
    GameConf.DS_Pad[ 0] = 3;  GameConf.DS_Pad[ 1] = 2;
    GameConf.DS_Pad[ 2] = 1;  GameConf.DS_Pad[ 3] = 0;
    GameConf.DS_Pad[ 4] = 4;  GameConf.DS_Pad[ 5] = 5;
    GameConf.DS_Pad[ 6] = 12;  GameConf.DS_Pad[ 7] = 12;
    GameConf.DS_Pad[ 8] = 15; GameConf.DS_Pad[ 9] = 16;
    GameConf.DS_Pad[10] = 14;  GameConf.DS_Pad[11] = 13;
   
    GameConf.sndLevel=127;
    GameConf.m_DisplayFPS=1; // 1 = yes
  }
}

void system_savecfg(char *cfg_name) {
  FILE *fd;
  
  fd = fopen(cfg_name, "wb");
  if (fd != NULL) {
    fwrite(&GameConf, 1, sizeof(GameConf),fd); 
    fclose(fd);
 }
}
