#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdint.h>

#include <ds2io.h>
#include <ds2_cpu.h>
#include <fs_api.h>
#include <ds2_malloc.h>

#include "myds2_timer.h"
#include "myds2_gfx.h"
#include "myds2_wav.h"

#include "main.h"
#include "menu.h"

#include "emu/Cartridge.h"
#include "emu/Database.h"
#include "emu/Palette.h"
#include "emu/ProSystem.h"
#include "emu/Sound.h"

#include "gfx/bgBottom_png.h"
#include "gfx/bgFileSel_png.h"
#include "gfx/bgTop_png.h"

#include "wav/clickNoQuit_wav.h"

#define DELAY_TCH 192000

FICA7800 proromlist[1024];  
unsigned int countpro=0, countfiles=0, ucFicAct=0;
unsigned int etatEmu;

unsigned char *filebuffer;
unsigned char *psound_buffer, *psndidx;

#define tchepres(a) \
   keyboard_data[GameConf.DS_Pad[a]] = 1;

unsigned char keyboard_data[19];
unsigned short bufVideo[320*312];        // Video flipping buffer
unsigned int gameCRC;                    // crc checksum of file  
uint video_height;                       // Actual video height
unsigned short capScreen[256*192];

static unsigned char timFirstTime=1;
unsigned short gamePal[256];

void dsInitScreenMain(void) {
}

void dsShowScreenEmu(void) {
}

void dsShowScreenMain(void) {
  // Load Up and down and wait for a key
  ds2pngshow(bgTop_png,bgTop_png_size, UP_SCREEN, 0,0, false);
  ds2pngshow(bgBottom_png, bgBottom_png_size, DOWN_SCREEN, 0,0, false);
  ds2_flipScreen(DUAL_SCREEN, 1);
}

void dsFreeEmu(void) {
  // Free buffer if needed
  if (filebuffer != 0)
    free(filebuffer);
  stopTimer(0);
}

void VsoundHandler(unsigned int arg) {
  psound_buffer++;
  if (psound_buffer>=&sound_buffer[SNDLENGTH]) psound_buffer=sound_buffer;
  processSound(psound_buffer,1);
}

void dsLoadGame(char *filename) {
  unsigned int index;
  
  // Free buffer if needed
  if (filebuffer != 0)
    free(filebuffer);
  stopTimer(0);

  // load card game if ok
  if (cartridge_Load(filename) != 0) {	
    // Initialize the virtual console emulation 
    database_Load(cartridge_digest);

    if (bios_Load("7800.rom")) {
      bios_enabled = true;
    }
  
    dsShowScreenEmu();
    prosystem_Reset();
        
    // Init palette
    for(index = 0; index < 256; index++) {
      word r = palette_data[(index * 3) + 0]>>3;
      word g = palette_data[(index * 3) + 1]>>3;
      word b = palette_data[(index * 3) + 2]>>3;
      gamePal[index] = RGB15(r, g, b);
    }
    
    // Init vars
    memset(keyboard_data,0,sizeof(keyboard_data));

    // Left difficulty switch defaults to off
    // Right difficulty swtich defaults to on
    keyboard_data[15] = 1;
    keyboard_data[16] = 0;
    GameConf.DS_Pad[ 0] = 3;  GameConf.DS_Pad[ 1] = 2;
    GameConf.DS_Pad[ 2] = 1;  GameConf.DS_Pad[ 3] = 0;
    GameConf.DS_Pad[ 4] = 4;  GameConf.DS_Pad[ 5] = 5;
    GameConf.DS_Pad[ 6] = 12;  GameConf.DS_Pad[ 7] = 12;
    GameConf.DS_Pad[ 8] = 15; GameConf.DS_Pad[ 9] = 16;
    GameConf.DS_Pad[10] = 14;  GameConf.DS_Pad[11] = 13;

    psound_buffer=sound_buffer;
    psndidx=sound_buffer;

    //Interrupt period =100icrosec
    if (timFirstTime) {
      initTimer(0, 45, VsoundHandler, 0); // 1/22050 -> 4.53e-5
      timFirstTime=0;
    }
    runTimer(0);
  }
}

bool dsWaitOnQuit(void) {
  bool bRet=false, bDone=false;
  unsigned int keys_pressed,i;
  unsigned int posdeb=0;
  char szName[32];
  struct key_buf input;
  unsigned short *screen = down_screen_addr;

  ds2pngshow(bgFileSel_png, bgFileSel_png_size, DOWN_SCREEN, 0,0, false);
  ds2_flipScreen(DOWN_SCREEN, 2);
  
  strcpy(szName,"Quit A7800DS2 ?");
  dsPrintValue(16-strlen(szName)/2,3,0,szName);
  sprintf(szName,"%s","A TO CONFIRM, B TO GO BACK");
  dsPrintValue(16-strlen(szName)/2,23,0,szName);
  strcpy(szName,"          YES          ");
  dsPrintValue(5,10+0,0,szName);
  strcpy(szName,"          NO           ");
  dsPrintValue(5,14+1,0,szName);
  for (i=0;i<256*192;i++) capScreen[i] = *(screen+i);

  while(!bDone) {
    for (i=0;i<256*192;i++) *(screen+i) = capScreen[i];
    strcpy(szName,"          YES          ");
    dsPrintValue(5,10+0,(posdeb == 0 ? 1 :  0),szName);
    strcpy(szName,"          NO           ");
    dsPrintValue(5,14+1,(posdeb == 2 ? 1 :  0),szName);
    ds2_flipScreen(DOWN_SCREEN, 2);
  
    // Check pad
    ds2_getrawInput(&input);keys_pressed=input.key;
    if (keys_pressed & KEY_UP) {
      if (posdeb) posdeb-=2;
    }
    if (keys_pressed & KEY_DOWN) {
      if (posdeb<1) posdeb+=2;
    }
    if (keys_pressed & KEY_A) {
      bRet = (posdeb ? false : true);
      bDone = true;
    }
    if (keys_pressed & KEY_B) {
      bDone = true;
    }
  }

  ds2pngshow(bgBottom_png, bgBottom_png_size, DOWN_SCREEN, 0,0, false);
  ds2_flipScreen(DOWN_SCREEN, 2);

  return bRet;
}

void dsDisplayFiles(unsigned int NoDebGame,u32 ucSel) {
  unsigned int ucBcl,ucGame;
  u8 maxLen;
  char szName[256];
  unsigned int i;
  unsigned short *screen = down_screen_addr;

  // Display all games if possible
  for (i=0;i<256*192;i++) *(screen+i) = capScreen[i];
  countfiles ? sprintf(szName,"%04d/%04d GAMES",1+ucSel+NoDebGame,countfiles) : sprintf(szName,"%04d/%04d FOLDERS",1+ucSel+NoDebGame,countpro);
  dsPrintValue(16-strlen(szName)/2,3,0,szName);
  dsPrintValue(31,5,0,(char *) (NoDebGame>0 ? "<" : " "));
  dsPrintValue(31,22,0,(char *) (NoDebGame+14<countpro ? ">" : " "));
  sprintf(szName,"%s","A TO SELECT A GAME, B TO GO BACK");
  dsPrintValue(16-strlen(szName)/2,23,0,szName);
  for (ucBcl=0;ucBcl<17; ucBcl++) {
    ucGame= ucBcl+NoDebGame;
    if (ucGame < countpro) {
      maxLen=strlen(proromlist[ucGame].filename);
      strcpy(szName,proromlist[ucGame].filename);
      if (maxLen>29) szName[29]='\0';
      if (proromlist[ucGame].directory) {
        sprintf(szName,"[%s]",proromlist[ucGame].filename);
        sprintf(szName,"%-29s",szName);
        dsPrintValue(0,5+ucBcl,(ucSel == ucBcl ? 1 :  0),szName);
      }
      else {
        sprintf(szName,"%-29s",strupr(szName));
        dsPrintValue(1,5+ucBcl,(ucSel == ucBcl ? 1 : 0),szName);
      }
    }
  }
  ds2_flipScreen(DOWN_SCREEN, 2);
}

unsigned int dsWaitForRom(void) {
  bool bDone=false, bRet=false;
  u32 ucHaut=0x00, ucBas=0x00,ucSHaut=0x00, ucSBas=0x00,romSelected= 0, firstRomDisplay=0,nbRomPerPage, uNbRSPage, uLenFic=0,ucFlip=0, ucFlop=0;
  char szName[64];
  struct key_buf input;
  unsigned int keys_pressed,i;

  ds2pngshow(bgFileSel_png, bgFileSel_png_size, DOWN_SCREEN, 0,0, false);
  unsigned short *screen = down_screen_addr;
  for (i=0;i<256*192;i++) capScreen[i] = *(screen+i);
  ds2_flipScreen(DOWN_SCREEN, 2);

  nbRomPerPage = (countpro>=17 ? 17 : countpro);
  uNbRSPage = (countpro>=5 ? 5 : countpro);
  if (ucFicAct>countpro-nbRomPerPage) {
    firstRomDisplay=countpro-nbRomPerPage;
    romSelected=ucFicAct-countpro+nbRomPerPage;
  }
  else {
    firstRomDisplay=ucFicAct;
    romSelected=0;
  }
  dsDisplayFiles(firstRomDisplay,romSelected);
  while (!bDone) {
    ds2_getrawInput(&input);keys_pressed=input.key;
    if (keys_pressed & KEY_UP) {
      if (!ucHaut) {
        ucFicAct = (ucFicAct>0 ? ucFicAct-1 : countpro-1);
        if (romSelected>uNbRSPage) { romSelected -= 1; }
        else {
          if (firstRomDisplay>0) { firstRomDisplay -= 1; }
          else {
            if (romSelected>0) { romSelected -= 1; }
            else {
              firstRomDisplay=countpro-nbRomPerPage;
              romSelected=nbRomPerPage-1;
            }
          }
        }
        ucHaut=0x01;
        dsDisplayFiles(firstRomDisplay,romSelected);
      }
      else {
        ucHaut++;
        if (ucHaut>DELAY_TCH) ucHaut=0;
      } 
    }
    else {
      ucHaut = 0;
    }  
    if (keys_pressed & KEY_DOWN) {
      if (!ucBas) {
        ucFicAct = (ucFicAct< countpro-1 ? ucFicAct+1 : 0);
        if (romSelected<uNbRSPage-1) { romSelected += 1; }
        else {
          if (firstRomDisplay<countpro-nbRomPerPage) { firstRomDisplay += 1; }
          else {
            if (romSelected<nbRomPerPage-1) { romSelected += 1; }
            else {
              firstRomDisplay=0;
              romSelected=0;
            }
          }
        }
        ucBas=0x01;
        dsDisplayFiles(firstRomDisplay,romSelected);
      }
      else {
        ucBas++;
        if (ucBas>DELAY_TCH) ucBas=0;
      } 
    }
    else {
      ucBas = 0;
    }  
    if (keys_pressed & KEY_R) {
      if (!ucSBas) {
        ucFicAct = (ucFicAct< countpro-nbRomPerPage ? ucFicAct+nbRomPerPage : countpro-nbRomPerPage);
        if (firstRomDisplay<countpro-nbRomPerPage) { firstRomDisplay += nbRomPerPage; }
        else { firstRomDisplay = countpro-nbRomPerPage; }
        if (ucFicAct == countpro-nbRomPerPage) romSelected = 0;
        ucSBas=0x01;
        dsDisplayFiles(firstRomDisplay,romSelected);
      }
      else {
        ucSBas++;
        if (ucSBas>DELAY_TCH) ucSBas=0;
      } 
    }
    else {
      ucSBas = 0;
    }  
    if (keys_pressed & KEY_L) {
      if (!ucSHaut) {
        ucFicAct = (ucFicAct> nbRomPerPage ? ucFicAct-nbRomPerPage : 0);
        if (firstRomDisplay>nbRomPerPage) { firstRomDisplay -= nbRomPerPage; }
        else { firstRomDisplay = 0; }
        if (ucFicAct == 0) romSelected = 0;
        ucSHaut=0x01;
        dsDisplayFiles(firstRomDisplay,romSelected);
      }
      else {
        ucSHaut++;
        if (ucSHaut>DELAY_TCH) ucSHaut=0;
      } 
    }
    else {
      ucSHaut = 0;
    }  
    if ( keys_pressed & KEY_B ) {
      bDone=true;
      while (keys_pressed & KEY_B) { ds2_getrawInput(&input);keys_pressed=input.key; }
    }

    if (keys_pressed & KEY_A) {
      if (!proromlist[ucFicAct].directory) {
        bRet=true;
        bDone=true;
      }
      else {
        chdir(proromlist[ucFicAct].filename);
        proFindFiles();
        ucFicAct = 0;
        nbRomPerPage = (countpro>=16 ? 16 : countpro);
        uNbRSPage = (countpro>=5 ? 5 : countpro);
        if (ucFicAct>countpro-nbRomPerPage) {
          firstRomDisplay=countpro-nbRomPerPage;
          romSelected=ucFicAct-countpro+nbRomPerPage;
        }
        else {
          firstRomDisplay=ucFicAct;
          romSelected=0;
        }
        dsDisplayFiles(firstRomDisplay,romSelected);
        while (keys_pressed & KEY_A) { ds2_getrawInput(&input);keys_pressed=input.key; }
      }
    }
    // Scroll la selection courante
    if (strlen(proromlist[ucFicAct].filename) > 29) {
      ucFlip++;
      if (ucFlip >= 8) {
        ucFlip = 0;
        uLenFic++;
        if ((uLenFic+29)>strlen(proromlist[ucFicAct].filename)) {
          ucFlop++;
          if (ucFlop >= 8) {
            uLenFic=0;
            ucFlop = 0;
          }
          else
            uLenFic--;
        }
        strncpy(szName,proromlist[ucFicAct].filename+uLenFic,29);
        szName[29] = '\0';
        dsPrintValue(1,5+romSelected,1,szName);
      }
    }
  }
  
  ds2pngshow(bgBottom_png, bgBottom_png_size, DOWN_SCREEN, 0,0, false);
  ds2_flipScreen(DOWN_SCREEN, 2);
  
  return bRet;
}

unsigned int dsWaitOnMenu(unsigned int actState) {
  unsigned int uState=A7800_PLAYINIT;
  unsigned int keys_pressed;
  bool bDone=false, romSel;
  int iTx,iTy;
  struct key_buf input;

  while (!bDone) {
    // wait for stylus
    ds2_getrawInput(&input);keys_pressed=input.key;
    if (keys_pressed & KEY_TOUCH) {
      iTx = input.x;
      iTy = input.y;
      if ((iTx>31) && (iTx<65) && (iTy>159) && (iTy<169))  { // 32,160  -> 64,168   quit
        play_wave(clickNoQuit_wav, clickNoQuit_wav_size);
        bDone=dsWaitOnQuit();
        if (bDone) uState=A7800_QUITSTDS;
      }
      if ((iTx>79) && (iTx<180) && (iTy>31) && (iTy<62)) {     // 80,32 -> 179,61 cartridge slot
        bDone=true; 
        // Find files in current directory and show it 
        proFindFiles();
        romSel=dsWaitForRom();
        if (romSel) { uState=A7800_PLAYINIT; 
          dsLoadGame(proromlist[ucFicAct].filename); }
        else { uState=actState; }
      }
    }
  }
  
  return uState;
}

void dsPrintValue(int x, int y, unsigned int isSelect, char *pchStr) {
  ds2drawtext(x*8,y*8,DOWN_SCREEN, pchStr,isSelect ? RGB15(5,14,19) : RGB15(31,31,31), isSelect ? RGB15(31,31,31) : RGB15(0,0,0), isSelect ? false : true);
}

void process_sound()  {
  // Do sound
  //assume 16-bit and stereo audio
  unsigned short *audio_buffer_addr;
  unsigned char *src;
  unsigned short *dst0;
  unsigned short *dst1;
  int n;

  if (ds2_checkAudiobuff() >= 2)
    return;

  //In fact, there is no neccessary to check whether the IO layer's audio buffers are busy or not, 
  //we updating audio not frequently as the example with DS2SDK did.
  audio_buffer_addr = (unsigned short*) ds2_getAudiobuff();
  if (NULL == audio_buffer_addr) { return; }

  src = (unsigned char*) psndidx;
  dst0 = audio_buffer_addr;
  dst1 = audio_buffer_addr + audio_samples_per_trans;

  //Fill the buffer
  n = 0;
  while (n++ < audio_samples_per_trans) {
    unsigned short value=(*src++)<<8;
    *dst0++ = value;
    *dst1++ = value;
    if (src>=&sound_buffer[SNDLENGTH]) src=sound_buffer;
  }
	  
  //Update audio  
  ds2_updateAudio();
  psndidx=src;
}

void bitmap_scale_fast(void) {
  unsigned short *buffer_scr = (unsigned short *) up_screen_addr;
//  unsigned short *buffer_scr = down_screen_addr;
  unsigned int W,H,ix,iy,x,y;
  x=0;
  y=maria_visibleArea.top;
  W=256;
  H=192;
  ix=(320<<16)/W;
  iy=(video_height<<16)/H;

  do   
  {
    unsigned short *buffer_mem=&bufVideo[(y>>16)*320+8*320];
    W=256; x=0;
    do {
      *buffer_scr++=gamePal[buffer_mem[x>>16]];
      x+=ix;
    } while (--W);
    y+=iy;
  } while (--H);
}

void dsMainLoop(void) {
  char fpsbuf[32];
  unsigned int keys_pressed,keys_touch=0, romSel;
  int iTx,iTy;
  struct key_buf input;
  unsigned int framecount = 0;
  unsigned int a7800_FPS = 0;
  unsigned int time1=0;


  while(etatEmu != A7800_QUITSTDS) {
    switch (etatEmu) {
      case A7800_MENUINIT:
        dsShowScreenMain();
        etatEmu = A7800_MENUSHOW;
        break;
        
      case A7800_MENUSHOW:
        etatEmu =  dsWaitOnMenu(A7800_MENUSHOW);
        break;
        
      case A7800_PLAYINIT:
        ds2timer_init();
        dsShowScreenEmu();
        etatEmu = A7800_PLAYGAME;
        break;
        
      case A7800_PLAYGAME:
        // Execute one frame
        prosystem_ExecuteFrame(keyboard_data);
        bitmap_scale_fast();

        {
          framecount++;
          if ((myClockTick - time1) >= MYTICKRATE) {
            time1 = myClockTick;
            a7800_FPS=framecount;
            framecount=0;
          }
          sprintf(fpsbuf,"%03d",a7800_FPS); //ds2drawstring(0,0, UP_SCREEN, fpsbuf, RGB15(31,31,31), RGB15(0,0,0), false);
        }

        // Read keys
        memset(keyboard_data,0,sizeof(keyboard_data));
        ds2_getrawInput(&input);keys_pressed=input.key;
        // if touch screen pressed
        if (keys_pressed & KEY_TOUCH) {
          if (!keys_touch) {
            keys_touch=1;
            iTx = input.x;
            iTy = input.y;
            if ((iTx>31) && (iTx<65) && (iTy>159) && (iTy<169))  { // 32,160  -> 64,168   quit
              stopTimer(0);
              play_wave(clickNoQuit_wav, clickNoQuit_wav_size);
              if (dsWaitOnQuit()) etatEmu=A7800_QUITSTDS;
              else { runTimer(0); }
            }
            else if ((iTx>71) && (iTx<106) && (iTy>159) && (iTy<169))  { // 72,160  -> 105,168   pause
              play_wave(clickNoQuit_wav, clickNoQuit_wav_size);
              tchepres(10);
            }
            else if ((iTx>190) && (iTx<225) && (iTy>159) && (iTy<169))  { // 191,160  -> 224,168   reset
              play_wave(clickNoQuit_wav, clickNoQuit_wav_size);
              tchepres(6);
            }
            else if ((iTx>141) && (iTx<176) && (iTy>159) && (iTy<169))  { // 142,160  -> 175,168   select
              play_wave(clickNoQuit_wav, clickNoQuit_wav_size);
              tchepres(11);
            }
            else if ((iTx>79) && (iTx<180) && (iTy>31) && (iTy<62)) {     // 80,32 -> 179,61 cartridge slot
              stopTimer(0);
              // Find files in current directory and show it 
              proFindFiles();
              romSel=dsWaitForRom();
              if (romSel) { etatEmu=A7800_PLAYINIT; dsLoadGame(proromlist[ucFicAct].filename); }
              else { runTimer(0); }
            }
          }
        }
        // else manage a7800 pad 
        else {
          keys_touch=0;
          if ( (keys_pressed & KEY_UP) ) { tchepres(0); } // UP
          if ( (keys_pressed & KEY_DOWN) ) { tchepres(1); } // DOWN
          if ( (keys_pressed & KEY_RIGHT) ) { tchepres(3); } // RIGHT
          if ( (keys_pressed & KEY_LEFT) ) { tchepres(2); } // LEFT

          if ( (keys_pressed & KEY_A) ) { tchepres(4); }  // BUTTON #1
          if ( (keys_pressed & KEY_B) ) { tchepres(5); }  // BUTTON #2
          
          if ( (keys_pressed & KEY_START) ) {tchepres(10);} // BUTTON PAUSE
          if ( (keys_pressed & KEY_SELECT) ) { tchepres(11); } // BUTTON SELECT
          if ( (keys_pressed & KEY_R) ) { tchepres(6); } // BUTTON RESET
          if ( (keys_pressed & KEY_L) ) { sprintf(fpsbuf,"%03d",a7800_FPS); ds2drawstring(0,0, UP_SCREEN, fpsbuf, RGB15(31,31,31), RGB15(0,0,0), false); } // Show FPS
        }

        ds2_flipScreen(UP_SCREEN, 0);
        process_sound();
        ds2timer_sync();
        break;
    }
  }
  prosystem_Close();
}


//----------------------------------------------------------------------------------
// Find files (a78 / bin) available
int a78Filescmp (const void *c1, const void *c2) {
  FICA7800 *p1 = (FICA7800 *) c1;
  FICA7800 *p2 = (FICA7800 *) c2;
  
  return strcmp (p1->filename, p2->filename);
}

void proFindFiles(void) {
  struct stat statbuf;
  DIR *current_dir;
  dirent *current_file;
  char *filenametmp;
  
  countpro = countfiles = 0;

  current_dir = opendir(".");
  if (current_dir) {
    while ( ( (current_file = readdir_ex(current_dir, &statbuf)) != NULL) && (countpro<1024)) {
      filenametmp = current_file->d_name;  
      if(S_ISDIR(statbuf.st_mode)) {
        if (!( (filenametmp[0] == '.') && (strlen(filenametmp) == 1))) {
          proromlist[countpro].directory = true;
          strcpy(proromlist[countpro].filename,filenametmp);
          countpro++;
        }
      }
      else {
        if (strlen(filenametmp)>4) {
          if ( (strcasecmp(strrchr(filenametmp, '.'), ".a78") == 0) )  {
            proromlist[countpro].directory = false;
            strcpy(proromlist[countpro].filename,filenametmp);
            countpro++;countfiles++;
          }
          if ( (strcasecmp(strrchr(filenametmp, '.'), ".bin") == 0) )  {
            proromlist[countpro].directory = false;
            strcpy(proromlist[countpro].filename,filenametmp);
            countpro++;countfiles++;
          }
        }
      }
    }
    closedir(current_dir);
  }
  if (countpro)
    qsort (proromlist, countpro, sizeof (FICA7800), a78Filescmp);
}


