#include <stdio.h>
#include <string.h>
#include <ds2_malloc.h>
#include <fs_api.h>

#include "myds2_gfx.h"

#include "lodepng.h"

#define RGB24_15(pixel) ((((*pixel) & 0xF8) << 7) |\
                        (((*(pixel+1)) & 0xF8) << 2) |\
                        (((*(pixel+2)) & 0xF8)>>3))

#define RGB16_15(pixel) ((((*pixel)>>10) & 0x1F) |\
						(((*pixel) & 0x1F) << 10) |\
						((*pixel) & 0x83E0))

extern const unsigned char font_map[128][8];

//font size 8*8
static inline void ds2drawfont(unsigned short *addr, unsigned short f_color, unsigned short b_color, unsigned char ch, bool transp) {
  unsigned char *dot_map;
  unsigned int j, k;
  unsigned char dot;
  unsigned short *dst;

  dot_map = (unsigned char*)font_map[ch&0x7F];

  if (transp) {
    for (j= 0; j < 8; j++)	{
      dot = *dot_map++;
      dst = addr + j*SCREEN_WIDTH;
      for (k = 0; k < 8; k++) {
        if (dot & (0x80>>k)) *dst = f_color;
        dst++;
      }
    }
  }
  else {
    for (j= 0; j < 8; j++)	{
      dot = *dot_map++;
      dst = addr + j*SCREEN_WIDTH;
      for (k = 0; k < 8; k++)
        *dst++ = (dot & (0x80>>k)) ? f_color : b_color;
    }
  }
}

void ds2drawstring(unsigned int x, unsigned int y, enum SCREEN_ID screen, char *string, unsigned short f_color, unsigned short b_color, bool transp) {
  unsigned short *scr_addr, *dst;

  if (screen & UP_MASK)
    scr_addr = (unsigned short *) up_screen_addr;
  else 
    scr_addr = (unsigned short *) down_screen_addr;

   if(x>= (32*8) || y>= (24*8)) return;

   while (*string)  {
     dst = scr_addr + (y)*SCREEN_WIDTH + x;
     ds2drawfont(dst, f_color, b_color, *string++,transp);
     
     x += 8;
     if (x>= 32*8) break;
   }
}

int ds2bmpread(char* filename, unsigned char* buf, unsigned int *width, unsigned int *height, unsigned int* type) {
  FILE* fp;
  BMPHEADER bmp_header;
  int	flag;
  unsigned  int bytepixel;
  unsigned int x, y, sx, sy, m, len;
  unsigned char *dest;
  signed int fpos;
  unsigned char st[54];
	
  fp= fopen(filename, "rb");

  if (fp == NULL)
    return BMP_ERR_OPENFAILURE;
	
  flag= fread(st, sizeof(st), 1, fp);
  if (!flag) {
    fclose(fp);
    return BMP_ERR_FORMATE;
  }
	
  bmp_header.bfType= *((u16*)st);
  bmp_header.bfSize= *((u16*)(st+2)) | *((u16*)(st+4));
  bmp_header.bfReserved0= *((u16*)(st+6));
  bmp_header.bfReserved1= *((u16*)(st+8));
  bmp_header.bfImgoffst= *((u16*)(st+10)) | *((u16*)(st+12));
  bmp_header.bfImghead.imHeadsize= *((u16*)(st+14)) | *((u16*)(st+16));
  bmp_header.bfImghead.imBitmapW= *((u16*)(st+18)) | *((u16*)(st+20));
  bmp_header.bfImghead.imBitmapH= *((u16*)(st+22)) | *((u16*)(st+24));
  bmp_header.bfImghead.imPlanes= *((u16*)(st+26));
  bmp_header.bfImghead.imBitpixel= *((u16*)(st+28));
  bmp_header.bfImghead.imCompess= *((u16*)(st+30)) | *((u16*)(st+32));
  bmp_header.bfImghead.imImgsize= *((u16*)(st+34)) | *((u16*)(st+36));
  bmp_header.bfImghead.imHres= *((u16*)(st+38)) | *((u16*)(st+40));
  bmp_header.bfImghead.imVres= *((u16*)(st+42)) | *((u16*)(st+44));
  bmp_header.bfImghead.imColnum= *((u16*)(st+46)) | *((u16*)(st+48));
  bmp_header.bfImghead.imImcolnum= *((u16*)(st+50)) | *((u16*)(st+52));

  if  (bmp_header.bfType != 0x4D42) {	//"BM"
    fclose(fp);
    return BMP_ERR_FORMATE;
  }

  if (bmp_header.bfImghead.imCompess != BI_RGB &&  bmp_header.bfImghead.imCompess != BI_BITFIELDS) {
    fclose(fp);
    return BMP_ERR_NEED_GO_ON;		//This funciton now not support...
  }

  bytepixel= bmp_header.bfImghead.imBitpixel >> 3;
  if (bytepixel < 2) {					//byte per pixel >= 2
    fclose(fp);
    return BMP_ERR_NEED_GO_ON;		//This funciton now not support...
  }

  *type = bytepixel;

//  x= *width;
//  y= *height;
  sx= bmp_header.bfImghead.imBitmapW;
  sy= bmp_header.bfImghead.imBitmapH;
  x=sx;
  y=sy;

  len = x < sx ? x : sx;
/*  if (y > sy)
    y= sy;
*/

  //BMP scan from down to up
  fpos= (signed int)bmp_header.bfImgoffst;
  dest= (unsigned char*)buf+(y-1)*x*4;
  for (m= 0; m < y; m++) {
    fseek(fp, fpos, SEEK_SET);
    fread(dest, 1, len*bytepixel, fp);
    fpos += ((sx*bytepixel+3)>>2)<<2;
    dest -= x*4;
  }

  fclose(fp);

/*
  if (x > sx)
    x= sx;
*/

  *width = x;
  *height= y;	

  return BMP_OK;
}

signed int ds2bmpshow(char *file,enum SCREEN_ID screen, unsigned int xc, unsigned int yc, bool flipscreen) {
  unsigned int bmp_type;
  unsigned char *bmp_buf;
  unsigned short *dst;	
  unsigned int x, y;
  unsigned int width, height;
  unsigned short *scr_addr;

  if (screen & UP_MASK)
    scr_addr = (unsigned short *) up_screen_addr;
  else 
    scr_addr = (unsigned short *) down_screen_addr;

  bmp_buf = (unsigned char*)malloc(SCREEN_WIDTH*SCREEN_HEIGHT*4);  // max size of bmp = screen size
  if (NULL == bmp_buf)
    return -1;

  memset(bmp_buf, 0, SCREEN_WIDTH*SCREEN_HEIGHT*4);
//  ds2_clearScreen(screen, 0);

  width = SCREEN_WIDTH;
  height = SCREEN_HEIGHT;
  if (ds2bmpread(file, bmp_buf, &width, &height, &bmp_type) !=BMP_OK ) {
    free((void*)bmp_buf);
    return -1;
  }

  if (bmp_type ==2) {		//2 bytes per pixel
    unsigned short *pt;
    for (y= 0; y< height; y++) {
      dst = (unsigned short*)scr_addr + (y+yc)*SCREEN_WIDTH +xc;
//      pt = (unsigned short*)bmp_buf + y*SCREEN_WIDTH*2;
      pt = (unsigned short*)bmp_buf + y*width*2;
      for (x= 0; x< width; x++) {
        *dst++= RGB16_15(pt);
        pt += 1;
      }
    }
  }
  else if(bmp_type ==3)	{ //3 bytes per pixel
     unsigned char *buff;
     for(y= 0; y< height; y++) {
       dst = (unsigned short*)scr_addr + (y+yc)*SCREEN_WIDTH +xc;
//       buff = bmp_buf + y*SCREEN_WIDTH*4;
       buff = bmp_buf + y*width*4;
       for(x= 0; x< width; x++) {
         *dst++= RGB24_15(buff);
         buff += 3;
       }
     }
  }
  else {
    free((void*)bmp_buf);
    return -1;
  }

  if (flipscreen) ds2_flipScreen(screen, 1);
  free((void*)bmp_buf);

  return 0;
}

signed int ds2bmptexload(char *file,ds2gfxtexture *inTexture,unsigned short inColor) {
  unsigned int bmp_type;
  unsigned char *bmp_buf;
  unsigned short *dst;	
  unsigned int x, y;
  unsigned int width, height;

  bmp_buf = (unsigned char*)malloc(SCREEN_WIDTH*SCREEN_HEIGHT*4);
  if (NULL == bmp_buf)
    return -1;

  memset(bmp_buf, 0, SCREEN_WIDTH*SCREEN_HEIGHT*4);

  width = SCREEN_WIDTH;
  height = SCREEN_HEIGHT;
  if (ds2bmpread(file, bmp_buf, &width, &height, &bmp_type) !=BMP_OK ) {
    free((void*)bmp_buf);
    return -1;
  }
  inTexture->width = width;
  inTexture->height = height;
  inTexture->transpcolor = inColor;

  if (bmp_type ==2) {		//2 bytes per pixel
    unsigned short *pt;
    for (y= 0; y< height; y++) {
      dst = (unsigned short*)inTexture->address + (y)*width;
      pt = (unsigned short*)bmp_buf + y*width*2;
      for (x= 0; x< width; x++) {
        *dst++= RGB16_15(pt);
        pt += 1;
      }
    }
  }
  else if(bmp_type ==3)	{ //3 bytes per pixel
     unsigned char *buff;
     for(y= 0; y< height; y++) {
       dst = (unsigned short*)inTexture->address + (y)*width;
       buff = bmp_buf + y*width*4;
       for(x= 0; x< width; x++) {
         *dst++= RGB24_15(buff);
         buff += 3;
       }
     }
  }
  else {
    free((void*)bmp_buf);
    return -1;
  }

  free((void*)bmp_buf);

  return 0;
}

signed int ds2pngread(unsigned char *stream,unsigned int streamsize, unsigned short* buf) {
  signed int err_ds2 = -1;
  unsigned short *dst;	
  unsigned int x, y;
  unsigned int width, height;
  unsigned char *image;
  size_t imagesize;
  LodePNG_Decoder decoder;

  LodePNG_Decoder_init(&decoder);
  BAG_LodePNG_decode(&decoder, &image, &imagesize, stream, streamsize); /*decode the png*/

  if (!decoder.error) { /* no error */
    err_ds2 = 0;
    
    width = decoder.infoPng.width;
    height = decoder.infoPng.height;

     // TODO, check number of bits
    unsigned char *pt;
    for (y= 0; y< height; y++) {
      dst = (unsigned short*)buf + y*width;
      pt = (unsigned char*) image + y*width*4;
      for (x= 0; x< width; x++) {
	int r = (*(pt+4 * x + 0)>>3); /*red*/
	int g = (*(pt+4 * x + 1)>>3) ;/*green*/
	int b = (*(pt+4 * x + 2)>>3);  /*blue*/		
        *dst++= RGB15(r,g,b);
      }
    }
  }

  if(image!=NULL) free(image);
            
  LodePNG_Decoder_cleanup(&decoder);

  return err_ds2;
}

signed int ds2pngshow(unsigned char *stream,unsigned int streamsize, enum SCREEN_ID screen, unsigned int xc, unsigned int yc, bool flipscreen) {
  signed int err_ds2 = -1;
  unsigned short *dst;	
  unsigned int x, y;
  unsigned int width, height;
  unsigned short *scr_addr;
  unsigned char *image;
  size_t imagesize;
  LodePNG_Decoder decoder;

  if (screen & UP_MASK)
    scr_addr = (unsigned short *) up_screen_addr;
  else 
    scr_addr = (unsigned short *) down_screen_addr;

  LodePNG_Decoder_init(&decoder);
  BAG_LodePNG_decode(&decoder, &image, &imagesize, stream, streamsize); /*decode the png*/

  if (!decoder.error) { /* no error */
    err_ds2 = 0;
    
    width = decoder.infoPng.width;
    height = decoder.infoPng.height;

     // TODO, check number of bits
    unsigned char *pt;
    for (y= 0; y< height; y++) {
      dst = (unsigned short*)scr_addr + (y+yc)*SCREEN_WIDTH +xc;
      pt = (unsigned char*) image + y*width*4;
      for (x= 0; x< width; x++) {
	int r = (*(pt+4 * x + 0)>>3); /*red*/
	int g = (*(pt+4 * x + 1)>>3) ;/*green*/
	int b = (*(pt+4 * x + 2)>>3);  /*blue*/		
        *dst++= RGB15(r,g,b);
      }
    }
    if (flipscreen) ds2_flipScreen(screen, 1);
  }

  if(image!=NULL) free(image);
            
  LodePNG_Decoder_cleanup(&decoder);

  return err_ds2;
}

void ds2rectfilldraw(enum SCREEN_ID screen, unsigned int inX, unsigned int inY, unsigned int inWidth, unsigned int inHeight, unsigned short inColor) {
  unsigned short *scr_addr;
  unsigned int i, j;

  if(((inX + inWidth) <= 0) || (inX >= 256)) return;
  if(((inY + inHeight) <= 0) || (inY >= 192)) return;

  if(inWidth < 0) {
    inX += inWidth;
    inWidth = -inWidth;
  }
  if(inHeight < 0) {
    inY += inHeight;
    inHeight = -inHeight;
   }
  if(inX < 0) {
    inWidth += inX;
    inX = 0;
  }
  if((inX + inWidth) > 256) inWidth = (256 - inX);
  if(inY < 0) {
    inHeight += inY;
    inY = 0;
  }
  if((inY + inHeight) > 192)  inHeight = (192 - inY);

  if (screen & UP_MASK)
    scr_addr = (unsigned short *) up_screen_addr;
  else 
    scr_addr = (unsigned short *) down_screen_addr;

  for(j = inY; (j < 192) && (j < (inY + inHeight)); j++) {
    for(i = inX; (i < 256) && (i < (inX + inWidth)); i++) {
	scr_addr[(j * 256) + i] = inColor;
    }
  }
}

const unsigned char fontdata8x8[] = {
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x3C,0x42,0x99,0xBD,0xBD,0x99,0x42,0x3C,0x3C,0x42,0x81,0x81,0x81,0x81,0x42,0x3C,
	0xFE,0x82,0x8A,0xD2,0xA2,0x82,0xFE,0x00,0xFE,0x82,0x82,0x82,0x82,0x82,0xFE,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x38,0x64,0x74,0x7C,0x38,0x00,0x00,
	0x80,0xC0,0xF0,0xFC,0xF0,0xC0,0x80,0x00,0x01,0x03,0x0F,0x3F,0x0F,0x03,0x01,0x00,
	0x18,0x3C,0x7E,0x18,0x7E,0x3C,0x18,0x00,0xEE,0xEE,0xEE,0xCC,0x00,0xCC,0xCC,0x00,
	0x00,0x00,0x30,0x68,0x78,0x30,0x00,0x00,0x00,0x38,0x64,0x74,0x7C,0x38,0x00,0x00,
	0x3C,0x66,0x7A,0x7A,0x7E,0x7E,0x3C,0x00,0x0E,0x3E,0x3A,0x22,0x26,0x6E,0xE4,0x40,
	0x18,0x3C,0x7E,0x3C,0x3C,0x3C,0x3C,0x00,0x3C,0x3C,0x3C,0x3C,0x7E,0x3C,0x18,0x00,
	0x08,0x7C,0x7E,0x7E,0x7C,0x08,0x00,0x00,0x10,0x3E,0x7E,0x7E,0x3E,0x10,0x00,0x00,
	0x58,0x2A,0xDC,0xC8,0xDC,0x2A,0x58,0x00,0x24,0x66,0xFF,0xFF,0x66,0x24,0x00,0x00,
	0x00,0x10,0x10,0x38,0x38,0x7C,0xFE,0x00,0xFE,0x7C,0x38,0x38,0x10,0x10,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1C,0x1C,0x1C,0x18,0x00,0x18,0x18,0x00,
	0x6C,0x6C,0x24,0x00,0x00,0x00,0x00,0x00,0x00,0x28,0x7C,0x28,0x7C,0x28,0x00,0x00,
	0x10,0x38,0x60,0x38,0x0C,0x78,0x10,0x00,0x40,0xA4,0x48,0x10,0x24,0x4A,0x04,0x00,
	0x18,0x34,0x18,0x3A,0x6C,0x66,0x3A,0x00,0x18,0x18,0x20,0x00,0x00,0x00,0x00,0x00,
	0x30,0x60,0x60,0x60,0x60,0x60,0x30,0x00,0x0C,0x06,0x06,0x06,0x06,0x06,0x0C,0x00,
	0x10,0x54,0x38,0x7C,0x38,0x54,0x10,0x00,0x00,0x18,0x18,0x7E,0x18,0x18,0x00,0x00,
	0x00,0x00,0x00,0x00,0x18,0x18,0x30,0x00,0x00,0x00,0x00,0x00,0x3E,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x18,0x18,0x00,0x00,0x00,0x04,0x08,0x10,0x20,0x40,0x00,0x00,
	0x38,0x4C,0xC6,0xC6,0xC6,0x64,0x38,0x00,0x18,0x38,0x18,0x18,0x18,0x18,0x7E,0x00,
	0x7C,0xC6,0x0E,0x3C,0x78,0xE0,0xFE,0x00,0x7E,0x0C,0x18,0x3C,0x06,0xC6,0x7C,0x00,
	0x1C,0x3C,0x6C,0xCC,0xFE,0x0C,0x0C,0x00,0xFC,0xC0,0xFC,0x06,0x06,0xC6,0x7C,0x00,
	0x3C,0x60,0xC0,0xFC,0xC6,0xC6,0x7C,0x00,0xFE,0xC6,0x0C,0x18,0x30,0x30,0x30,0x00,
	0x78,0xC4,0xE4,0x78,0x86,0x86,0x7C,0x00,0x7C,0xC6,0xC6,0x7E,0x06,0x0C,0x78,0x00,
	0x00,0x00,0x18,0x00,0x00,0x18,0x00,0x00,0x00,0x00,0x18,0x00,0x00,0x18,0x18,0x30,
	0x1C,0x38,0x70,0xE0,0x70,0x38,0x1C,0x00,0x00,0x7C,0x00,0x00,0x7C,0x00,0x00,0x00,
	0x70,0x38,0x1C,0x0E,0x1C,0x38,0x70,0x00,0x7C,0xC6,0xC6,0x1C,0x18,0x00,0x18,0x00,
	0x3C,0x42,0x99,0xA1,0xA5,0x99,0x42,0x3C,0x38,0x6C,0xC6,0xC6,0xFE,0xC6,0xC6,0x00,
	0xFC,0xC6,0xC6,0xFC,0xC6,0xC6,0xFC,0x00,0x3C,0x66,0xC0,0xC0,0xC0,0x66,0x3C,0x00,
	0xF8,0xCC,0xC6,0xC6,0xC6,0xCC,0xF8,0x00,0xFE,0xC0,0xC0,0xFC,0xC0,0xC0,0xFE,0x00,
	0xFE,0xC0,0xC0,0xFC,0xC0,0xC0,0xC0,0x00,0x3E,0x60,0xC0,0xCE,0xC6,0x66,0x3E,0x00,
	0xC6,0xC6,0xC6,0xFE,0xC6,0xC6,0xC6,0x00,0x7E,0x18,0x18,0x18,0x18,0x18,0x7E,0x00,
	0x06,0x06,0x06,0x06,0xC6,0xC6,0x7C,0x00,0xC6,0xCC,0xD8,0xF0,0xF8,0xDC,0xCE,0x00,
	0x60,0x60,0x60,0x60,0x60,0x60,0x7E,0x00,0xC6,0xEE,0xFE,0xFE,0xD6,0xC6,0xC6,0x00,
	0xC6,0xE6,0xF6,0xFE,0xDE,0xCE,0xC6,0x00,0x7C,0xC6,0xC6,0xC6,0xC6,0xC6,0x7C,0x00,
	0xFC,0xC6,0xC6,0xC6,0xFC,0xC0,0xC0,0x00,0x7C,0xC6,0xC6,0xC6,0xDE,0xCC,0x7A,0x00,
	0xFC,0xC6,0xC6,0xCE,0xF8,0xDC,0xCE,0x00,0x78,0xCC,0xC0,0x7C,0x06,0xC6,0x7C,0x00,
	0x7E,0x18,0x18,0x18,0x18,0x18,0x18,0x00,0xC6,0xC6,0xC6,0xC6,0xC6,0xC6,0x7C,0x00,
	0xC6,0xC6,0xC6,0xEE,0x7C,0x38,0x10,0x00,0xC6,0xC6,0xD6,0xFE,0xFE,0xEE,0xC6,0x00,
	0xC6,0xEE,0x3C,0x38,0x7C,0xEE,0xC6,0x00,0x66,0x66,0x66,0x3C,0x18,0x18,0x18,0x00,
	0xFE,0x0E,0x1C,0x38,0x70,0xE0,0xFE,0x00,0x3C,0x30,0x30,0x30,0x30,0x30,0x3C,0x00,
	0x60,0x60,0x30,0x18,0x0C,0x06,0x06,0x00,0x3C,0x0C,0x0C,0x0C,0x0C,0x0C,0x3C,0x00,
	0x18,0x3C,0x66,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xFF,
	0x30,0x30,0x18,0x00,0x00,0x00,0x00,0x00,0x00,0x3C,0x06,0x3E,0x66,0x66,0x3C,0x00,
	0x60,0x7C,0x66,0x66,0x66,0x66,0x7C,0x00,0x00,0x3C,0x66,0x60,0x60,0x66,0x3C,0x00,
	0x06,0x3E,0x66,0x66,0x66,0x66,0x3E,0x00,0x00,0x3C,0x66,0x66,0x7E,0x60,0x3C,0x00,
	0x1C,0x30,0x78,0x30,0x30,0x30,0x30,0x00,0x00,0x3E,0x66,0x66,0x66,0x3E,0x06,0x3C,
	0x60,0x7C,0x76,0x66,0x66,0x66,0x66,0x00,0x18,0x00,0x38,0x18,0x18,0x18,0x18,0x00,
	0x0C,0x00,0x1C,0x0C,0x0C,0x0C,0x0C,0x38,0x60,0x60,0x66,0x6C,0x78,0x6C,0x66,0x00,
	0x38,0x18,0x18,0x18,0x18,0x18,0x18,0x00,0x00,0xEC,0xFE,0xFE,0xFE,0xD6,0xC6,0x00,
	0x00,0x7C,0x76,0x66,0x66,0x66,0x66,0x00,0x00,0x3C,0x66,0x66,0x66,0x66,0x3C,0x00,
	0x00,0x7C,0x66,0x66,0x66,0x7C,0x60,0x60,0x00,0x3E,0x66,0x66,0x66,0x3E,0x06,0x06,
	0x00,0x7E,0x70,0x60,0x60,0x60,0x60,0x00,0x00,0x3C,0x60,0x3C,0x06,0x66,0x3C,0x00,
	0x30,0x78,0x30,0x30,0x30,0x30,0x1C,0x00,0x00,0x66,0x66,0x66,0x66,0x6E,0x3E,0x00,
	0x00,0x66,0x66,0x66,0x66,0x3C,0x18,0x00,0x00,0xC6,0xD6,0xFE,0xFE,0x7C,0x6C,0x00,
	0x00,0x66,0x3C,0x18,0x3C,0x66,0x66,0x00,0x00,0x66,0x66,0x66,0x66,0x3E,0x06,0x3C,
	0x00,0x7E,0x0C,0x18,0x30,0x60,0x7E,0x00,0x0E,0x18,0x0C,0x38,0x0C,0x18,0x0E,0x00,
	0x18,0x18,0x18,0x00,0x18,0x18,0x18,0x00,0x70,0x18,0x30,0x1C,0x30,0x18,0x70,0x00,
	0x00,0x00,0x76,0xDC,0x00,0x00,0x00,0x00,0x10,0x28,0x10,0x54,0xAA,0x44,0x00,0x00,
};

void ds2drawtext(unsigned int x, unsigned int y, enum SCREEN_ID screen, char *text, unsigned short f_color, unsigned short b_color, bool transp) {
  unsigned int i,l;
  unsigned short *tempBuffer;

  if (screen & UP_MASK)
    tempBuffer = (unsigned short *) up_screen_addr;
  else 
    tempBuffer = (unsigned short *) down_screen_addr;

  tempBuffer=tempBuffer+x+y*256;

  if (transp) {
    for (i=0;i<strlen(text);i++) {
      for (l=0;l<8;l++) {
        tempBuffer[l*256+0]=(fontdata8x8[((text[i])*8)+l]&0x80)?f_color:tempBuffer[l*256+0];
        tempBuffer[l*256+1]=(fontdata8x8[((text[i])*8)+l]&0x40)?f_color:tempBuffer[l*256+1];
        tempBuffer[l*256+2]=(fontdata8x8[((text[i])*8)+l]&0x20)?f_color:tempBuffer[l*256+2];
        tempBuffer[l*256+3]=(fontdata8x8[((text[i])*8)+l]&0x10)?f_color:tempBuffer[l*256+3];
        tempBuffer[l*256+4]=(fontdata8x8[((text[i])*8)+l]&0x08)?f_color:tempBuffer[l*256+4];
        tempBuffer[l*256+5]=(fontdata8x8[((text[i])*8)+l]&0x04)?f_color:tempBuffer[l*256+5];
        tempBuffer[l*256+6]=(fontdata8x8[((text[i])*8)+l]&0x02)?f_color:tempBuffer[l*256+6];
        tempBuffer[l*256+7]=(fontdata8x8[((text[i])*8)+l]&0x01)?f_color:tempBuffer[l*256+7];
      }
      tempBuffer+=8;
    } 
  }
  else {
    for (i=0;i<strlen(text);i++) {
      for (l=0;l<8;l++) {
        tempBuffer[l*256+0]=(fontdata8x8[((text[i])*8)+l]&0x80)?f_color:b_color;
        tempBuffer[l*256+1]=(fontdata8x8[((text[i])*8)+l]&0x40)?f_color:b_color;
        tempBuffer[l*256+2]=(fontdata8x8[((text[i])*8)+l]&0x20)?f_color:b_color;
        tempBuffer[l*256+3]=(fontdata8x8[((text[i])*8)+l]&0x10)?f_color:b_color;
        tempBuffer[l*256+4]=(fontdata8x8[((text[i])*8)+l]&0x08)?f_color:b_color;
        tempBuffer[l*256+5]=(fontdata8x8[((text[i])*8)+l]&0x04)?f_color:b_color;
        tempBuffer[l*256+6]=(fontdata8x8[((text[i])*8)+l]&0x02)?f_color:b_color;
        tempBuffer[l*256+7]=(fontdata8x8[((text[i])*8)+l]&0x01)?f_color:b_color;
      }
      tempBuffer+=8;
    } 
  }
}

void ds2drawtexwithbg(unsigned int inX, unsigned int inY, enum SCREEN_ID screen, ds2gfxtexture *inTexture) {
  unsigned short *tempBuffer;

  if (screen & UP_MASK)
    tempBuffer = (unsigned short *) up_screen_addr;
  else 
    tempBuffer = (unsigned short *) down_screen_addr;

  if(inTexture == NULL) return;

  unsigned short *tempTexture = (unsigned short *) inTexture->address;
  unsigned short colortex;

  unsigned int x, y, i, j;
  for(j = 0, y = inY; (j < inTexture->height) && (y < 192); j++, y++) {
    for(i = 0, x = inX; (i < inTexture->width) && (x < 256); i++, x++) {
      colortex = tempTexture[(j * inTexture->width) + i];
      if (colortex != inTexture->transpcolor) tempBuffer[(y * 256) + x] = colortex;
    }
  }
}

bool ds2savetextga(const char* inPath, ds2gfxtexture* inTexture) {
  if ((inPath == NULL) || (inTexture == NULL) || (inTexture->address == NULL))
    return false;

  FILE* tempFile = fopen(inPath, "wb");
  if (tempFile == NULL)
    return false;

  unsigned char tga_ident_size = 0;
  unsigned char  tga_color_map_type = 0;
  unsigned char  tga_image_type = 2;
  unsigned short tga_color_map_start = 0;
  unsigned short tga_color_map_length = 0;
  unsigned char  tga_color_map_bpp = 0;
  unsigned short tga_origin_x = 0;
  unsigned short tga_origin_y = 0;
  short tga_width = (short) inTexture->width;
  short tga_height = (short) inTexture->height;
  unsigned char tga_bpp = 24;
  unsigned char tga_descriptor = 0x20;

  fwrite(&tga_ident_size, 1, 1, tempFile);
  fwrite(&tga_color_map_type, 1, 1, tempFile);
  fwrite(&tga_image_type, 1, 1, tempFile);
  fwrite(&tga_color_map_start, 2, 1, tempFile);
  fwrite(&tga_color_map_length, 2, 1, tempFile);
  fwrite(&tga_color_map_bpp, 1, 1, tempFile);
  fwrite(&tga_origin_x, 2, 1, tempFile);
  fwrite(&tga_origin_y, 2, 1, tempFile);
  fwrite(&tga_width, 2, 1, tempFile);
  fwrite(&tga_height, 2, 1, tempFile);
  fwrite(&tga_bpp, 1, 1, tempFile);
  fwrite(&tga_descriptor, 1, 1, tempFile);

  unsigned int i;
  unsigned char tempColor[3];
  unsigned short * tempTexPtr = (unsigned short *) inTexture->address;
  for (i = 0; i < (tga_width * tga_height); i++) {
    tempColor[0]  = ((tempTexPtr[i] >> 8) & 0xF8);
    tempColor[0] |= (tempColor[0] >> 5);
    tempColor[1]  = ((tempTexPtr[i] >> 3) & 0xFC);
    tempColor[1] |= (tempColor[1] >> 6);
    tempColor[2]  = ((tempTexPtr[i] << 3) & 0xF8);
    tempColor[2] |= (tempColor[2] >> 5);
    fwrite(&tempColor[2], 1, 1, tempFile);
    fwrite(&tempColor[1], 1, 1, tempFile);
    fwrite(&tempColor[0], 1, 1, tempFile);
  }
  fclose(tempFile);

  return true;
}

