#ifndef __myds2_gfx_h__
#define __myds2_gfx_h__

#include <ds2io.h>

typedef struct _pixelmapheader{
	unsigned int		imHeadsize;	//Bitmap information header size
	unsigned int		imBitmapW;	//bitmap width in pixel
	unsigned int		imBitmapH;	//bitmap height in pixel
	unsigned short		imPlanes;	//bitmap planes numbers, must be set to 1
	unsigned short		imBitpixel;	//bits per pixel
	unsigned int		imCompess;	//compress method
	unsigned int		imImgsize;	//image size, times of 4-byte
	unsigned int		imHres;		//horizontal resolution, pixel/metel
	unsigned int		imVres;		//vertical resolution, pixel/metel
	unsigned int		imColnum;	//number of colors in color palette, 0 to exp(2)
	unsigned int		imImcolnum;	//important colors numbers used
} IMAGEHEADER;


typedef struct _bitmapfileheader{
	unsigned short		bfType;		//BMP file types
	unsigned int		bfSize;		//BMP file size(Not the pixel image size)
	unsigned short		bfReserved0;//reserved area0
	unsigned short		bfReserved1;//reserved area1
	unsigned int		bfImgoffst;	//pixel data area offset
	IMAGEHEADER bfImghead;
} BMPHEADER;

//compression method
/* Value Identified by 	Compression method 		Comments
*	0 		BI_RGB 			none 				Most common
*	1 		BI_RLE8 		RLE 8-bit/pixel 	Can be used only with 8-bit/pixel bitmaps
*	2 		BI_RLE4 		RLE 4-bit/pixel 	Can be used only with 4-bit/pixel bitmaps
*	3 		BI_BITFIELDS 	Bit field 			Can be used only with 16 and 32-bit/pixel bitmaps.
*	4 		BI_JPEG 		JPEG 				The bitmap contains a JPEG image
*	5 		BI_PNG 			PNG 				The bitmap contains a PNG image
*/
#define BI_RGB			0
#define BI_RLE8			1
#define BI_RLE4			2
#define BI_BITFIELDS 	3
#define BI_JPEG			4
#define BI_PNG			5

//error message
#define BMP_OK				0
#define BMP_ERR_OPENFAILURE	1
#define BMP_ERR_FORMATE		2
#define BMP_ERR_NOTSUPPORT	3
#define BMP_ERR_NEED_GO_ON	4


#define FILEOPENCHECK(fp)		(fp!=NULL)

typedef struct {
  void*          address;
  unsigned int width, height;
  unsigned short transpcolor;
} ds2gfxtexture;

extern void ds2drawstring(unsigned int x, unsigned int y, enum SCREEN_ID screen, char *string, unsigned short f_color, unsigned short b_color, bool transp);
extern int ds2bmpread(char* filename, unsigned char* buf, unsigned int *width, unsigned int *height, unsigned int* type);
extern signed int ds2bmpshow(char *file,enum SCREEN_ID screen, unsigned int xc, unsigned int yc, bool flipscreen);
extern signed int ds2bmptexload(char *file,ds2gfxtexture *inTexture,unsigned short inColor);

extern signed int ds2pngread(unsigned char *stream,unsigned int streamsize, unsigned short* buf);
extern signed int ds2pngshow(unsigned char *stream,unsigned int streamsize, enum SCREEN_ID screen, unsigned int xc, unsigned int yc, bool flipscreen);

extern void ds2drawtext(unsigned int x, unsigned int y, enum SCREEN_ID screen, char *text, unsigned short f_color, unsigned short b_color, bool transp);
extern void ds2drawtexwithbg(unsigned int inX, unsigned int inY, enum SCREEN_ID screen, ds2gfxtexture *inTexture);
extern bool ds2savetextga(const char* inPath, ds2gfxtexture* inTexture);

#endif

